/*
 * Copyright (c) 2002-2009 syntevo GmbH. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of syntevo GmbH nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.syntevo.q.gui.layout;

import java.awt.*;

/**
 * @author Thomas Singer
 */
public abstract class AbstractLayout implements LayoutManager2 {

	// Abstract ===============================================================

	protected abstract void doAddLayoutComponent(Component comp, Object constraints);

	protected abstract void doRemoveLayoutComponent(Component comp);

	protected abstract Dimension getPreferredLayoutSize(Insets insets);

	protected abstract Dimension getMinimumLayoutSize(Insets insets);

	protected abstract void doLayoutContainer(Dimension size, Insets insets);

	// Implemented ============================================================

	public final void addLayoutComponent(Component component, Object constraints) {
		synchronized (component.getTreeLock()) {
			doAddLayoutComponent(component, constraints);
		}
	}

	public final void removeLayoutComponent(Component comp) {
		synchronized (comp.getTreeLock()) {
			doRemoveLayoutComponent(comp);
		}
	}

	public final Dimension preferredLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			return getPreferredLayoutSize(target.getInsets());
		}
	}

	public final Dimension minimumLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			return getMinimumLayoutSize(target.getInsets());
		}
	}

	public final void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			doLayoutContainer(target.getSize(), target.getInsets());
		}
	}

	public final Dimension maximumLayoutSize(Container target) {
		return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
	}

	public final float getLayoutAlignmentX(Container target) {
		return 0.0f;
	}

	public final float getLayoutAlignmentY(Container target) {
		return 0.0f;
	}

	public final void addLayoutComponent(String name, Component comp) {
		throw new UnsupportedOperationException();
	}
}
