package com.syntevo.plugin.jira.transport;

import java.io.*;
import java.text.*;
import java.util.*;

/**
 * @author syntevo GmbH
 */
class JiraDebugLoggingLogStream extends OutputStream {

	// Constants ==============================================================

	private static final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

	// Fields =================================================================

	private final String prefix;
	private final ByteArrayOutputStream buffer;
	private final PrintWriter log;

	// Setup ==================================================================

	public JiraDebugLoggingLogStream(PrintWriter log, String prefix) {
		this.log = log;
		this.buffer = new ByteArrayOutputStream(2048);
		this.prefix = prefix;
	}

	// Implemented ============================================================

	public void write(int b) throws IOException {
		buffer.write(b);
		flushBuffer(false);
	}

	public void close() throws IOException {
		flushBuffer(true);
	}

	public void flush() throws IOException {
		flushBuffer(true);
	}

	public void write(byte[] b, int off, int len) throws IOException {
		buffer.write(b, off, len);
		flushBuffer(false);
	}

	// Accessing ==============================================================

	public void flushBuffer(boolean force) {
		if (!force && buffer.size() < 1024) {
			return;
		}
		if (buffer.size() > 0) {
			log.println(DATE_FORMAT.format(new Date()) + " " + prefix + new String(buffer.toByteArray()));
			log.flush();
		}
		buffer.reset();
	}
}