package com.syntevo.plugin.jira.transport;

import java.io.*;
import java.net.*;
import java.security.*;
import java.security.cert.*;
import javax.net.ssl.*;

/**
 * @author syntevo GmbH
 */
final class JiraHttpsKeyManager implements X509KeyManager {

	// Fields =================================================================

	private final JiraConnection connection;

	private PrivateKey privateKey;
	private java.security.cert.Certificate certificate;

	// Setup ==================================================================

	public JiraHttpsKeyManager(JiraConnection connection) {
		this.connection = connection;
	}

	// Implemented ============================================================

	public String[] getClientAliases(String location, Principal[] principals) {
		return new String[0];
	}

	public String chooseClientAlias(String[] strings, Principal[] principals, Socket socket) {
		if (!checkClientCertificate("")) {
			return null;
		}

		return certificate.getPublicKey().getAlgorithm();
	}

	public String[] getServerAliases(String location, Principal[] principals) {
		return new String[0];
	}

	public String chooseServerAlias(String location, Principal[] principals, Socket socket) {
		return null;
	}

	public X509Certificate[] getCertificateChain(String location) {
		if (!checkClientCertificate("")) {
			return new X509Certificate[0];
		}

		return new X509Certificate[] {(X509Certificate)certificate};
	}

	public PrivateKey getPrivateKey(String location) {
		if (!checkClientCertificate("")) {
			return null;
		}

		return privateKey;
	}

	// Utils ==================================================================

	private boolean checkClientCertificate(String location) {
		if (certificate != null && privateKey != null) {
			return true;
		}

		final JiraHttpsClientCertificate clientCertificate = connection.getSSLClientCertificate(location);
		try {
			final KeyStore ks = KeyStore.getInstance("PKCS12");
			final String password = clientCertificate.getKeyPassword();
			final File keyFile = clientCertificate.getKeyFile();
			final char[] passwordChars = password != null ? password.toCharArray() : null;
			final FileInputStream inputStream = new FileInputStream(keyFile);
			try {
				ks.load(inputStream, passwordChars);
			}
			finally {
				inputStream.close();
			}

			privateKey = (PrivateKey)ks.getKey(ks.aliases().nextElement(), passwordChars);
			certificate = ks.getCertificate(ks.aliases().nextElement());
			return true;
		}
		catch (GeneralSecurityException ex) {
			connection.showError(ex.getMessage(), ex);
			return false;
		}
		catch (IOException ex) {
			connection.showError(ex.getMessage(), ex);
			return false;
		}
	}
}