package com.syntevo.plugin.tagmultiple;

import java.util.*;

import org.tmatesoft.svn.core.*;
import org.tmatesoft.svn.core.wc.*;
import org.tmatesoft.svn.core.wc2.*;

import com.syntevo.openapi.smartsvn.command.*;
import com.syntevo.openapi.smartsvn.file.*;
import com.syntevo.openapi.smartsvn.tagbranch.*;
import com.syntevo.openapi.smartsvn.thread.*;

/**
 * @author syntevo GmbH
 */
final class TagMultipleProjectRootsSvnCommand implements ISvnCommand {

	// Fields =================================================================

	private final String tagName;
	private final Set<? extends SvnDirectory> svnDirectories;
	private final boolean fixExternalsRevisions;
	private final SVNURL fixExternalsRevisionExclusion;
	private final SvnTagBranchLayouts svnTagBranchLayouts;
	private final String commitMessage;

	// Setup ==================================================================

	public TagMultipleProjectRootsSvnCommand(String tagName, Set<? extends SvnDirectory> svnDirectories, boolean fixExternalsRevisions, SVNURL fixExternalsRevisionExclusion, SvnTagBranchLayouts svnTagBranchLayouts, String commitMessage) {
		this.tagName = tagName;
		this.svnDirectories = svnDirectories;
		this.fixExternalsRevisions = fixExternalsRevisions;
		this.fixExternalsRevisionExclusion = fixExternalsRevisionExclusion;
		this.svnTagBranchLayouts = svnTagBranchLayouts;
		this.commitMessage = commitMessage;
	}

	// Implemented ============================================================

	public void perform(SvnClients svnClients, ProgressViewer progressViewer) throws SVNException {
		final Map<SvnDirectory, SVNURL> directoryToLayout = new HashMap<SvnDirectory, SVNURL>();
		for (final SvnDirectory svnDirectory : svnDirectories) {
			final SvnTagBranchLayout layout = svnTagBranchLayouts.getTrunkTagBranchLayout(SVNURL.parseURIDecoded(svnDirectory.getURLString()));
			if (layout == null) {
				throw new SVNException(SVNErrorMessage.create(SVNErrorCode.UNKNOWN, "No tag branch layout found for " + svnDirectory.getPath()));
			}

			final SVNURL tagLocation = layout.getTagLocation(tagName);
			if (tagLocation == null) {
				throw new SVNException(SVNErrorMessage.create(SVNErrorCode.UNKNOWN, "Can't determine tag location for " + svnDirectory.getPath()));
			}

			directoryToLayout.put(svnDirectory, tagLocation);
		}

		final SvnOperationFactory operationsFactory = svnClients.createOperationsFactory();

		double i = 0;
		for (final Iterator<? extends SvnDirectory> it = svnDirectories.iterator(); it.hasNext(); i++) {
			final SvnDirectory svnDirectory = it.next();
			final SVNURL tagLocation = directoryToLayout.get(svnDirectory);
			progressViewer.setMessage("Tagging " + svnDirectory.getPath().getFile().getAbsolutePath() + " ...");
			progressViewer.setProgress(i / svnDirectories.size());

			final SvnRemoteCopy cp = operationsFactory.createRemoteCopy();
			cp.setExternalsHandler(new TagMultipleProjectRootsExternalsHandler(fixExternalsRevisions, fixExternalsRevisionExclusion));
			cp.setSingleTarget(SvnTarget.fromURL(tagLocation));
			cp.setMove(false);
			cp.setFailWhenDstExists(true);
			cp.setMakeParents(true);
			cp.setCommitMessage(commitMessage);
			cp.setCommitHandler(svnClients.getCommitHandler());

			final SvnTarget target = SvnTarget.fromFile(svnDirectory.getPath().getFile(), SVNRevision.WORKING);
			cp.addCopySource(SvnCopySource.create(target, SVNRevision.WORKING));
			cp.run();
		}
	}
}
